#!/bin/bash

MAINDIR="$CHARM_DIR"
HOSTFILE="${MAINDIR}/ansiblecharmhosts"
GITDIR="${MAINDIR}/mycharmplaybook"
KEYDIR="${MAINDIR}/.ssh"
KEYFILE="${KEYDIR}/git_id_rsa"

@when 'config.changed.git_deploy_key'
function installkey() {
    # install the SSH key
    juju-log -l 'INFO' "installkey called"
    mkdir -p "$KEYDIR"
    config-get git_deploy_key > "$KEYFILE"
    chmod 600 "$KEYFILE"
    chmod 700 "$KEYDIR"
    juju-log -l 'INFO' "key installed"
    set_flag 'reclone'
}

@when_any 'reclone' 'config.changed.git_repo' 'config.changed.git_branch'
function clone() {
    clear_flag 'reclone'
    repo="$(config-get git_repo)"
    git_path="$(which git)"
    # rm -rf myplaybook
    # git clone $repo myplaybook
    juju-log -l 'INFO' "clone called $git_path $repo in $(pwd)"
    status-set maintenance "Cloning git repository at $repo"
    rm -rf "$GITDIR"
    branch="$(config-get git_branch)"
    if [ -n "$(config-get git_deploy_key)" ]; then
        export GIT_SSH_COMMAND="ssh -i $KEYFILE"
    fi
    git clone -b "$branch" "$repo" "$GITDIR" || return 1
    juju-log -l 'INFO' "repository cloned"
    git -C "$GITDIR" submodule init && git -C "$GITDIR" submodule update && juju-log -l 'INFO' "submodules updated"
    set_flag 'ansiblehosts'
}

@when_any 'ansiblehosts' 'config.changed.ansible_inventory_hostgroup' 'config.changed.ansible_inventory_dir'
function createansiblehosts() {
    clear_flag 'ansiblehosts'
    hostgroup="$(config-get ansible_inventory_hostgroup)"
    if [ -z "$hostgroup" ]; then
        hostgroup="local"
    fi
    echo "[${hostgroup}]" > $HOSTFILE
    echo "localhost ansible_connection=local" >> $HOSTFILE
    juju-log -l 'INFO' "ansible hosts file created"
    cat $HOSTFILE

    inventorydir="$(config-get ansible_inventory_dir)"
    if [ -n "$inventorydir" ]; then
        linkedinventory="${GITDIR}/${inventorydir}/ansiblecharmhosts" 
        rm -f "$linkedinventory"
        ln -s -v "$HOSTFILE" "$linkedinventory" 
        juju-log -l 'INFO' "ansible hosts linked in inventory directory $linkedinventory"
    fi
    set_flag 'run-playbook'
}

function run_playbook() {
    # run the playbook
    ansible_path="$(which ansible-playbook)"
    playbook_yaml="$(config-get ansible_playbook_yaml)"
    juju-log -l 'INFO' run_playbook called $ansible_path
    if [ "$(status-get)" == "blocked" ]; then
        juju-log -l 'WARN' "run_playbook called but status is blocked"
        return 1
    fi
    status-set maintenance "Running playbook"
    flags=""
    if [ "$(config-get ansible_playbook_become)" == "True" ]; then
        flags="${flags} -b "
    fi
    if [ -n "$(config-get ansible_playbook_tags)" ]; then
        flags="${flags} -t $(config-get ansible_playbook_tags) "
    fi
    inventorydir="$(config-get ansible_inventory_dir)"
    if [ -n "$inventorydir" ]; then
        flags="${flags} -i ${GITDIR}/${inventorydir}/ansiblecharmhosts "
    else
        flags="${flags} -i $HOSTFILE "
    fi
    juju-log -l 'INFO' "ansible-playbook $flags ${GITDIR}/$playbook_yaml"
    export HOME="$CHARM_DIR"
    ansible-playbook $flags "${GITDIR}/$playbook_yaml" && return 0
}

@when_any 'run-playbook' 'config.changed.ansible_playbook_yaml' 'config.changed.ansible_playbook_become' 'config.changed.ansible_playbook_tags'
run_playbook_wrap() {
    clear_flag 'run-playbook'
    clear_flag 'config.changed.git_repo'
    clear_flag 'config.changed.git_branch'
    clear_flag 'config.changed.git_deploy_key'
    clear_flag 'config.changed.ansible_inventory_hostgroup'
    clear_flag 'config.changed.ansible_inventory_dir'
    clear_flag 'config.changed.ansible_playbook_yaml'
    clear_flag 'config.changed.ansible_playbook_become'
    clear_flag 'config.changed.ansible_playbook_tags'
    if run_playbook; then
        status-set active
    else
        status-set blocked "Error running playbook"
    fi
}

@when_not 'config.set.git_repo'
function no_repo() {
    status-set blocked "Missing git_repo"
}

@when_any 'eval-update' 'config.changed.check_update_eval'
function evaluate_on_update() {
    clear_flag 'eval-update'
    juju-log -l 'INFO' "evaluating check_update_eval"

    eval "$(config-get check_update_eval)"
    updateeval=$?

    juju-log -l 'WARNING' "update_eval returned $updateeval"

    if [ "$updateeval" == 0 ]; then
        juju-log -l 'INFO' "playbook not executed"
    else
        set_flag 'reclone'
    fi
}

#
# Hooks
#

@hook 'update-status'
function update() {
    juju-log -l 'INFO' update-status called 
    if [ "$(status-get)" == "blocked" ]; then
        juju-log -l 'WARN' "update called but status is blocked"
        return
    fi
    set_flag 'eval-update'
}

@hook 'juju-info-relation-changed'
function relation_changed() {
    juju-log  -l 'INFO' "juju-info-relation-changed called"
    set_flag 'reclone'
}



